@@ -1,18 +1,17 @@
 import argparse
 import inspect
-import logging
 import re
 import sys
 
-import app_skellington
+from . import NoCommandSpecified, app_container
 from ._bootstrap import _bootstrap_logger
-from . import app_container
 
 # If explicit fail is enabled, any command with at least one unknown
 # argument will be rejected entirely. If not enabled, unknown arguments
 # will be ignored.
 EXPLICIT_FAIL_ON_UNKNOWN_ARGS = True
 
+
 class CommandTree:
     """
     Command-line interface to hold a menu of commands. You can register
@@ -42,20 +41,21 @@ class CommandTree:
 
         ./scriptname --option="value" [submenu] [command]
 
-        is different than
+        is different from
 
         ./scriptname [submenu] [command] --option="value"
 
     in that option is being applied to the application in the first example and
-    applied to the refresh_datasets command (under the nhsn command group) in
+    applied to the command (under the submenu command group) in
     the second. In the same way the -h, --help options print different docs
     depending on where the help option was passed.
     """
+
     def __init__(self):
         self.root_parser = argparse.ArgumentParser()
-        self.submenu_param = None # submenu_param is the variable name
-                                  # of the root submenu argument, i.e. the arg
-                                  # in root_parser which selects the submenu.
+        self.submenu_param = None  # submenu_param is the variable name
+        # of the root submenu argument, i.e. the arg
+        # in root_parser which selects the submenu.
         self.entries = {}
         # NOTE(MG) Implementation note:
         # CommandTree uses only one of these internal structures (i.e. mutually exclusive),
@@ -66,13 +66,15 @@ class CommandTree:
         self._single_command = None
 
     def print_tree(self):
-        raise NotImplemented
+        raise NotImplementedError
 
     def add_argument(self, *args, **kwargs):
         """
         Adds an argument to the root parser.
         """
-        _bootstrap_logger.info('adding argument to root parser: %s and %s', args, kwargs)
+        _bootstrap_logger.info(
+            "adding argument to root parser: %s and %s", args, kwargs
+        )
         self.root_parser.add_argument(*args, **kwargs)
 
     def init_submenu(self, param_name, is_required=False):
@@ -80,41 +82,25 @@ class CommandTree:
         Creates a root-level submenu with no entries. SubMenu node is
         returned which can have submenus and commands attached to it.
         """
-        # NOTE(MG) Fix below strategizes whether to pass in 'required'
-        # paremter to ArgumentParser.add_subparsers()
-        # which was added in in Python3.7.
-        # Must also be written into SubMenu.create_submenu.
-        func_args = {
-            'dest': param_name,
-            'metavar': param_name,
-            'required': is_required
-        }
-        if (
-            sys.version_info.major == 3
-            and sys.version_info.minor <= 6
-        ):
-            if is_required:
-                _bootstrap_logger.warn('Unable to enforce required submenu: Requires >= Python 3.7')
-            del func_args['required']
+        func_args = {"dest": param_name, "metavar": param_name, "required": is_required}
 
         # Creates an argument as a slot in the underlying argparse.
-        subparsers = self.root_parser.add_subparsers(
-            **func_args
-        )
+        subparsers = self.root_parser.add_subparsers(**func_args)
 
         submenu = SubMenu(self, subparsers, param_name)
-        submenu.submenu_path = ''
+        submenu.submenu_path = ""
         submenu.var_name = param_name
 
-        _bootstrap_logger.info('Initialized root-level submenu: Parameter = \'%s\'', param_name)
+        _bootstrap_logger.info(
+            "Initialized root-level submenu: Parameter = '%s'", param_name
+        )
         self.entries[param_name] = submenu
         self.submenu_param = param_name
 
-        return submenu 
+        return submenu
 
     def register_command(
-        self, func, cmd_name=None, func_signature=None,
-        docstring=None
+        self, func, cmd_name=None, func_signature=None, docstring=None
     ):
         """
         When no submenu functionality is desired, this links a single
@@ -128,7 +114,7 @@ class CommandTree:
             pass
             # print('func is method')
         else:
-            raise Exception('bad value passed in for function')
+            raise Exception("bad value passed in for function")
 
         if not cmd_name:
             # safe try/except
@@ -145,47 +131,42 @@ class CommandTree:
 
         # help is displayed next to the command in the submenu enumeration or
         # list of commands:
-        help_text = HelpGenerator.generate_help_from_sig(docstring)
+        HelpGenerator.generate_help_from_sig(docstring)
         # description is displayed when querying help for the specific command:
-        description_text = HelpGenerator.generate_description_from_sig(docstring)
+        HelpGenerator.generate_description_from_sig(docstring)
         # end copy-paste from SubMenu.register_command
 
-        # begin copy-paste then editted from SubMenu.register_command
-        # For each paramter in the function create an argparse argument in
+        # begin copy-paste then edited from SubMenu.register_command
+        # For each parameter in the function create an argparse argument in
         # the child ArgumentParser created for this menu entry:
         for key in params:
-            if key == 'self':
+            if key == "self":
                 continue
             param = params[key]
 
-            if '=' in str(param):
+            if "=" in str(param):
                 if param.default is None:
-                    helptext = 'default provided'
+                    helptext = "default provided"
                 else:
                     helptext = "default = '{}'".format(param.default)
                 self.root_parser.add_argument(
-                    key,
-                    help=helptext,
-                    nargs='?',
-                    default=param.default
+                    key, help=helptext, nargs="?", default=param.default
                 )
             else:
-                helptext = 'required'
-                self.root_parser.add_argument(
-                    key,
-                    help=helptext)
+                helptext = "required"
+                self.root_parser.add_argument(key, help=helptext)
 
         # Build the CommandEntry structure
         cmd = CommandEntry()
         cmd.argparse_node = self.root_parser
-        cmd.cmd_name = cmd_name 
+        cmd.cmd_name = cmd_name
         cmd.func_signature = sig
         # cmd.func_ref = None
         cmd.callback = func
 
         registered_name = cmd_name
-        _bootstrap_logger.info('registered command: %s', registered_name)
-        # end copy-paste then editted from SubMenu.register_command
+        _bootstrap_logger.info("registered command: %s", registered_name)
+        # end copy-paste then edited from SubMenu.register_command
 
         self._cmd_tree_is_single_command = True
         self._single_command = cmd
@@ -209,33 +190,39 @@ class CommandTree:
             #             'failed to parse arguments: explicitly failing to be safe')
             #         return False, False
 
-            if hasattr(pargs, 'usage'):
+            if hasattr(pargs, "usage"):
                 pass
                 # print('found usage in app_skellington')
 
             return pargs, unk, True
 
         # Note: SystemExit is raised when '-h' argument is supplied.
-        except SystemExit as ex:
+        except SystemExit:
             return None, None, False
 
     def run_command(self, args=None):
+        """
+        Args:
+            args (list[str]): Direct from STDIN
+        Raises:
+            NoCommandSpecified for invalid command.
+        """
         args, unk, success = self.parse(args)
         if not success:
-            _bootstrap_logger.info('cli - SystemExit: Perhaps user invoked --help')
+            _bootstrap_logger.info("cli - SystemExit: Perhaps user invoked --help")
             return
 
         if args is False and unk is False:
-            _bootstrap_logger.error('cli - Failed parsing args.')
+            _bootstrap_logger.error("cli - Failed parsing args.")
             return False
-        _bootstrap_logger.info('cli - Received args from shell: %s', args)
+        _bootstrap_logger.info("cli - Received args from shell: %s", args)
 
         args = vars(args)
 
         cmd = self._lookup_command(args)
         if cmd is None:
-            _bootstrap_logger.critical('cli - Failed to find command.')
-            return False
+            _bootstrap_logger.critical("cli - Failed to find command.")
+            raise NoCommandSpecified
 
         return self._invoke_command(cmd, args)
 
@@ -246,31 +233,41 @@ class CommandTree:
         # the CommandTree with no SubMenu (submenu will be disabled
         # in this case):
         if self._cmd_tree_is_single_command:
-            assert self._cmd_tree_is_single_command is True, 'corrupt data structure in CommandMenu'
-            assert self._entries is None, 'corrupt data structure in CommandMenu'
-            assert isinstance(self._single_command, CommandEntry), 'corrupt data structure in CommandMenu'
+            assert (
+                self._cmd_tree_is_single_command is True
+            ), "corrupt data structure in CommandMenu"
+            assert self._entries is None, "corrupt data structure in CommandMenu"
+            assert isinstance(
+                self._single_command, CommandEntry
+            ), "corrupt data structure in CommandMenu"
             return self._single_command
 
         # There is at least one submenu we need to go down:
         else:
 
-            assert self._single_command is None, 'corrupt data structure in CommandMenu'
-            assert self._cmd_tree_is_single_command == False, 'corrupt data structure in CommandMenu'
+            assert self._single_command is None, "corrupt data structure in CommandMenu"
+            assert (
+                self._cmd_tree_is_single_command is False
+            ), "corrupt data structure in CommandMenu"
 
             # Key or variable name used by argparse to store the submenu options
-            argparse_param = self.submenu_param # e.g.: submenu_root
+            argparse_param = self.submenu_param  # e.g.: submenu_root
             submenu = self.entries[argparse_param]
 
             while True:
                 if argparse_param not in keys:
-                    print('root menu parameter not found in args:', argparse_param)
-                    input('<broken>')
+                    print("root menu parameter not found in args:", argparse_param)
+                    input("<broken>")
 
                 val = args.get(argparse_param)
-                _bootstrap_logger.debug('cli - argparse command is \'{}\' = {}'.format(argparse_param, val))
+                _bootstrap_logger.debug(
+                    "cli - argparse command is '{}' = {}".format(argparse_param, val)
+                )
 
                 lookup = submenu.entries.get(val)
-                _bootstrap_logger.debug('cli - lookup, entries[{}] = {}'.format(val, lookup))
+                _bootstrap_logger.debug(
+                    "cli - lookup, entries[{}] = {}".format(val, lookup)
+                )
 
                 # pop value
                 del args[argparse_param]
@@ -283,7 +280,7 @@ class CommandTree:
                     # return self._invoke_command(lookup, args)
 
                 else:
-                    raise app_container.NoCommandSpecified('No command specified.')
+                    raise app_container.NoCommandSpecified("No command specified.")
 
     def _invoke_command(self, cmd, args):
         command_to_be_invoked = cmd.callback
@@ -296,16 +293,17 @@ class CommandTree:
             if param.name in args:
                 func_args.append(args[param.name])
 
-        _bootstrap_logger.info('cli - function: %s', func)
-        _bootstrap_logger.info('cli - function args: %s', func_args)
+        _bootstrap_logger.info("cli - function: %s", func)
+        _bootstrap_logger.info("cli - function args: %s", func_args)
         return command_to_be_invoked(*func_args)
 
     def _get_subparser(self):
         return self.root_parser._subparsers._actions[1]
 
+
 class SubMenu:
     def __init__(self, parent, subparsers_obj, name):
-        self.parent = parent # Reference to root CommandTree
+        self.parent = parent  # Reference to root CommandTree
         self.subparsers_obj = subparsers_obj
         self.name = name
         self.submenu_path = None
@@ -313,8 +311,7 @@ class SubMenu:
         self.entries = {}
 
     def register_command(
-        self, func, cmd_name=None, func_signature=None,
-        docstring=None
+        self, func, cmd_name=None, func_signature=None, docstring=None
     ):
         """
         Registers a command as an entry in this submenu. Provided function is
@@ -345,7 +342,7 @@ class SubMenu:
         elif inspect.ismethod(func):
             pass
         else:
-            raise Exception('bad value passed in for function')
+            raise Exception("bad value passed in for function")
 
         if not cmd_name:
             # TODO(MG) Safer sanitation
@@ -369,56 +366,46 @@ class SubMenu:
         # Entry in local argparse._SubParsersAction
         # type = ArgumentParser
         child_node = self.subparsers_obj.add_parser(
-            cmd_name, # Note: cmd_name here will be the VALUE
-                      # passed into the argparse arg VARIABLE NAME
-                      # created when the SubMenu/argparse.add_subparsers()
-                      # was created.
+            cmd_name,  # Note: cmd_name here will be the VALUE
+            # passed into the argparse arg VARIABLE NAME
+            # created when the SubMenu/argparse.add_subparsers()
+            # was created.
             help=help_text,
-            description=description_text
+            description=description_text,
         )
 
         # For each paramter in the function create an argparse argument in
         # the child ArgumentParser created for this menu entry:
         for key in params:
-            if key == 'self':
+            if key == "self":
                 continue
             param = params[key]
 
-            if '=' in str(param):
+            if "=" in str(param):
                 if param.default is None:
-                    helptext = 'default provided'
+                    helptext = "default provided"
                 else:
                     helptext = "default = '{}'".format(param.default)
                 child_node.add_argument(
-                    key,
-                    help=helptext,
-                    nargs='?',
-                    default=param.default
+                    key, help=helptext, nargs="?", default=param.default
                 )
             else:
-                helptext = 'required'
-                child_node.add_argument(
-                    key,
-                    help=helptext
-                )
+                helptext = "required"
+                child_node.add_argument(key, help=helptext)
 
         # Build the CommandEntry structure
         cmd = CommandEntry()
         cmd.argparse_node = child_node
-        cmd.cmd_name = cmd_name 
+        cmd.cmd_name = cmd_name
         cmd.func_signature = sig
         # cmd.func_ref = None
         cmd.callback = func
 
-        registered_name = '{}.{}'.format(
-            self.submenu_path,
-            cmd_name)
-        _bootstrap_logger.info('cli - registered command: %s', registered_name)
+        registered_name = "{}.{}".format(self.submenu_path, cmd_name)
+        _bootstrap_logger.info("cli - registered command: %s", registered_name)
         self.entries[cmd_name] = cmd
 
-    def create_submenu( 
-        self, var_name, cmd_entry_name=None, is_required=False
-    ):
+    def create_submenu(self, var_name, cmd_entry_name=None, is_required=False):
         """
         Creates a child-submenu.
 
@@ -443,52 +430,39 @@ class SubMenu:
         # Create an entry in self's submenu:
         # type = ArgumentParser
         entry_node = self.subparsers_obj.add_parser(
-            cmd_entry_name,
-            help='sub-submenu help',
-            description='sub-sub description')
+            cmd_entry_name, help="sub-submenu help", description="sub-sub description"
+        )
 
-        # NOTE(MG) Fix below strategizes whether to pass in 'required'
-        # paremter to ArgumentParser.add_subparsers()
-        # which was added in in Python3.7.
+        # NOTE(MG) Fix for Python>=3.7,
+        # argparse.ArgumentParser added 'required' argument.
         # Must also be written into CommandTree.init_submenu
-        func_args = {
-            'dest': var_name,
-            'metavar': var_name,
-            'required': is_required
-        }
-        if (
-            sys.version_info.major == 3
-            and sys.version_info.minor <= 6
-        ):
+        func_args = {"dest": var_name, "metavar": var_name, "required": is_required}
+        if sys.version_info.major == 3 and sys.version_info.minor < 7:
             if is_required:
-                _bootstrap_logger.warn('Unable to enforce required submenu: Requires >= Python 3.7')
-            del func_args['required']
+                _bootstrap_logger.warn(
+                    "Unable to enforce required submenu: Requires >= Python 3.7"
+                )
+            del func_args["required"]
+        # END fix for Python<3.7
+
         # Turn entry into a submenu of it's own:
         # type = _SubParsersAction
-        subp_node = entry_node.add_subparsers(
-            **func_args
-        )
-    
-        submenu = SubMenu(
-            self.parent,
-            subp_node,
-            cmd_entry_name
-        )
+        subp_node = entry_node.add_subparsers(**func_args)
+
+        submenu = SubMenu(self.parent, subp_node, cmd_entry_name)
 
         submenu.var_name = var_name
 
-        submenu.submenu_path = '{}.{}'.format(self.submenu_path, cmd_entry_name)
+        submenu.submenu_path = "{}.{}".format(self.submenu_path, cmd_entry_name)
         submenu_name = submenu.submenu_path
 
-        _bootstrap_logger.info('cli - registered submenu: %s', submenu_name)
+        _bootstrap_logger.info("cli - registered submenu: %s", submenu_name)
         self.entries[cmd_entry_name] = submenu
         return submenu
 
     def __repr__(self):
-        return 'SubMenu({})<{}>'.format(
-            self.name,
-            ','.join(['cmds'])
-        )
+        return "SubMenu({})<{}>".format(self.name, ",".join(["cmds"]))
+
 
 class CommandEntry:
     """
@@ -504,18 +478,20 @@ class CommandEntry:
     arguments into argparse options (creating the documentation also). Similary,
     it can convert from argparse options into a function call.
     """
+
     def __init__(self):
         self.argparse_node = None
 
-        self.cmd_name = None # Don't think we need. And needs to be changed
-                             # from SubMenu
+        self.cmd_name = None  # Don't think we need. And needs to be changed
+        # from SubMenu
         self.menu_path = None
         self.func_signature = None
         self.func_ref = None
         self.callback = None
 
     def __repr__(self):
-        return 'CommandEntry<{}>'.format(self.cmd_name)
+        return "CommandEntry<{}>".format(self.cmd_name)
+
 
 class HelpGenerator:
     def __init__(self):
@@ -525,14 +501,14 @@ class HelpGenerator:
     def generate_help_from_sig(doctext):
         """
         The 'help' text is displayed next to the command when enumerating
-        the submenu commands. 
+        the submenu commands.
         """
-        if doctext == None:
+        if doctext is None:
             return doctext
-        regex = '(.*?)[.?!]'
+        regex = "(.*?)[.?!]"
         match = re.match(regex, doctext, re.MULTILINE | re.DOTALL)
         if match:
-            return match.group(1) + '.'
+            return match.group(1) + "."
         return doctext
 
     @staticmethod
@@ -541,11 +517,10 @@ class HelpGenerator:
         The 'description' paragraph is provided when the user requests help
         on a specific command.
         """
-        if doctext == None:
+        if doctext is None:
             return doctext
-        regex = '(.*?)[.?!]'
+        regex = "(.*?)[.?!]"
         match = re.match(regex, doctext, re.MULTILINE | re.DOTALL)
         if match:
-            return match.group(1) + '.'
+            return match.group(1) + "."
         return doctext
-
